@@ -36,7 +36,8 @@
 #define BROWN_NOISE           1
 #define WINNER_PROJECT        2
 #define LOG_NOISE             3
-#define TOTAL_SPECIALS        4
+#define CYCLE_COUNT           4
+#define TOTAL_SPECIALS        5
 
 /*
  * Get the next item in the environment
@@ -263,6 +264,9 @@ static void bubble_samples( SEXP workspace, int dofcount, int samplecount, doubl
 }
 
 /*
+ * WARNING: name of this function is exported, cannot be same as other items inside this
+ * package namespace (else it will conflict).
+ *
  * func -- something generated by the expression() call in R
  *         when evaluated, it will return a single scalar which is the `score'
  *
@@ -279,13 +283,14 @@ static void bubble_samples( SEXP workspace, int dofcount, int samplecount, doubl
  * maxiter -- the count of iterations (actually counts backwards to zero then exits)
  */
 
-SEXP gafit( SEXP func, SEXP env, SEXP thermal_R, SEXP maxiter_R, SEXP samplecount_R, SEXP step_R )
+SEXP gafit_C( SEXP func, SEXP env, SEXP thermal_R, SEXP maxiter_R, SEXP samplecount_R, SEXP step_R, SEXP tolerance_R )
 {
 	int maxiter;
 	int degrees_of_freedom = 0;
     int samplecount;
 	double thermal;
 	double stepsize;
+	double tolerance;
 	SEXP t1;
 	SEXP t2;
 	SEXP p;
@@ -294,6 +299,7 @@ SEXP gafit( SEXP func, SEXP env, SEXP thermal_R, SEXP maxiter_R, SEXP samplecoun
 	SEXP index;
 	int t3;
 	int i;
+	int cycle_count;
 
 	if( EXPRSXP == TYPEOF( func ))
 	{
@@ -307,6 +313,7 @@ SEXP gafit( SEXP func, SEXP env, SEXP thermal_R, SEXP maxiter_R, SEXP samplecoun
 	if( samplecount < 3 ) error( "Too few samples" );
 	thermal = asReal( thermal_R );
 	stepsize = asReal( step_R );
+	tolerance = asReal( tolerance_R );
 /*
  * Scan through the environment and get all the values of all the items,
  * these can be logicals, integers, reals or complex. Their mode will never
@@ -403,8 +410,17 @@ SEXP gafit( SEXP func, SEXP env, SEXP thermal_R, SEXP maxiter_R, SEXP samplecoun
  * (i.e. index 0) is not noisy and contains the original parameters plus an evaluation
  * of func within those parameters. This is the start of the iterations.
  */
+	cycle_count = 0;
 	while( maxiter-- )
 	{
+/*
+ * Check for a score within acceptable tolerance (presume best is at the top).
+ */
+		if( tolerance > REAL( workspace )[ SAMPLE_SCORE ]) { break; }
+		cycle_count++;
+/*
+ * Now sort the stack, see if something better comes along.
+ */
 		bubble_samples( workspace, degrees_of_freedom, samplecount, thermal );
 /*
  * Copy the best into the worst (encourages samples to try to be best)
@@ -425,9 +441,16 @@ SEXP gafit( SEXP func, SEXP env, SEXP thermal_R, SEXP maxiter_R, SEXP samplecoun
  * Save the best results in the environment variables so that they
  * can be returned to the user
  */
-	bubble_samples( workspace, degrees_of_freedom, samplecount, 0 );
-	bubble_samples( workspace, degrees_of_freedom, samplecount, 0 );
+	if( !maxiter )
+	{
+		bubble_samples( workspace, degrees_of_freedom, samplecount, 0 );
+		bubble_samples( workspace, degrees_of_freedom, samplecount, 0 );
+	}
 	inject_sample_env( workspace, degrees_of_freedom, indirect, index, 0 );
+/*
+ * Save the cycle_count in case the end user wants to see it.
+ */
+	REAL( workspace )[ CYCLE_COUNT ] = cycle_count;
 	UNPROTECT( 4 );
 	return( workspace );
 }
